%% CMM_CLEAN.M A simple point cloud editing program
% Greg Johnson, 2003-2004
% Please credit CMM_CLEAN, Greg Johnson, and the University of Manchester
% for all usage
%
% Try these keys in the figure window:
% f flatten data
% a cycle aspoect ratio
% m cycle marker size
% c set colour limits
% r square off against outline
% u undoes last crop (yahoo!)
% w write data to disk
% o show outline on screen
% z toggle zoom mode (LMB zooms instead of normal)
% q quit without saving
% 1,2,3 4 change view
% LMB drag - crops data
% SHIFT-LMB drag - nudge data up (in z)
% CTRL-LMB drag - nudge data down (in z)
%
% Requires Matlab 7 (Release 14) or later (support for subfunctions)
% and the fit_plane function (from GJ) and also read_three_columns.m
%
% Modified 16/2/2005
% GJ - added nudge and undo.  Cool beans, eh?

function cmm_clean(varargin)
persistent app
if nargin==0
  % startup
  app=[];
  app.data.fnamedata=uigetfile('{*.*}','Select the data file');
  if isequal(app.data.fnamedata,0)
    fprintf('Cancelled...\n');
    return
  end
  [app.data.x,app.data.y,app.data.z]=read_three_columns(app.data.fnamedata);
  %app.data.x=app.data.x(1:5:end);
  %app.data.y=app.data.y(1:5:end);
  %app.data.z=app.data.z(1:5:end);
  
  
  %The following lines are to be uncommented for dealing with badly scaled
  %data ONLY.
  %app.data.x=app.data.x*0.1;
  %app.data.y=app.data.y*0.4;
  %app.data.z=app.data.z*0.001;
  
  sfSetupVariables
  sfSetupFigure
else
  if nargin==3
    disp('Ahh!  You want me to look at that data?  OK...')
    app.data.x=varargin{1};
    app.data.y=varargin{2};
    app.data.z=varargin{3};
    sfSetupVariables
    sfSetupFigure
  else
    switch varargin{1}
      case 'mouse'
        sfMouseDrag
      case 'key'
        sfKeyPress
      otherwise
        disp('That sort of thing shouldn''t be allowed...');
    end
  end
end
%%%%%%%%%%%%%%%%%%%

%% sfKeyPress
  function sfKeyPress
    app.view.lastkeypress=get(app.view.h_fig,'currentcharacter');  
    switch app.view.lastkeypress
      case 'f'
        sfFlattenData
      case 'a'
        sfCycleAspectRatio
      case 'm'
        sfCycleMarkerSize
      case 'r'
        sfSquareData
      case 'o'
        sfToggleOutline
      case 'c'
        sfSetColourLimits
      case 'u'
        sfUndoLastCrop
      case 'z'
        sfToggleZoomIn
      case 'Z'
        sfZoomOut
      case 'w'
        sfWriteData
      case 'q'
        sfQuit
      case {'1','2','3','4','5','6','7','8'}
        app.view.viewaxis_selector=str2double(app.view.lastkeypress);
        view(app.view.viewvector(app.view.viewaxis_selector,:));
        set(gca,'cameraupvector',app.view.cameraupvector(app.view.viewaxis_selector,:));
      case ','
        % bit of a kludge to permit multiple nudges without reselecting...
        app.data.z(app.data.selection)=app.data.z(app.data.selection)-0.01;
        sfReplotData;
      case '.'
        app.data.z(app.data.selection)=app.data.z(app.data.selection)+0.01;
        sfReplotData;
      case '<'
        app.data.z(app.data.selection)=app.data.z(app.data.selection)-0.001;
        sfReplotData;
      case '>'
        app.data.z(app.data.selection)=app.data.z(app.data.selection)+0.001;
        sfReplotData;
      otherwise
        %disp('Haven''t the faintest idea what you''re asking...');
    end
  end

%% sfMouseDrag
  function sfMouseDrag
    pmask=logical([1 1 1]);
    pmask(app.view.viewaxis_selector)=0;
    seltype=get(gcf,'selectiontype');
    %if strcmp(seltype,'normal')
    %  disp('LMB - pass to zoom')
    %  return
    %end

    point1 = get(app.view.h_ax,'CurrentPoint');    % button down detected
    finalRect = rbbox;                   % return figure units
    point2 = get(app.view.h_ax,'CurrentPoint');    % button up detected
    set(gcf,'pointer','arrow')
    point1 = point1(1,pmask);            % extract x and y
    point2 = point2(1,pmask);
    p1 = min(point1,point2);             % calculate locations
    offset = abs(point1-point2);         % and dimensions
    x = [p1(1) p1(1)+offset(1) p1(1)+offset(1) p1(1) p1(1)];
    y = [p1(2) p1(2) p1(2)+offset(2) p1(2)+offset(2) p1(2)];

    u1=x(1);u2=x(3);
    v1=y(1);v2=y(3);

    switch app.view.viewaxis_selector
      case 1
        a=find(app.data.y>u1 & app.data.y<u2);
        b=find(app.data.z>v1 & app.data.z<v2);
      case 2
        a=find(app.data.x>u1 & app.data.x<u2);
        b=find(app.data.z>v1 & app.data.z<v2);
      case 3
        a=find(app.data.x>u1 & app.data.x<u2);
        b=find(app.data.y>v1 & app.data.y<v2);
      otherwise
        a=[];b=[];
    end

    c=intersect(a,b);

    if strcmp(seltype,'normal')
      % LMB, should we crop or zoom?
      if app.view.zoomon
        % zoom
        app.view.axlimits=[u1 u2 v1 v2];
        set(gca,'xlim',app.view.axlimits(1:2),'ylim',app.view.axlimits(3:4));

        drawnow
      else
        % crop
        app.data.undoable.x=app.data.x(c);
        app.data.undoable.y=app.data.y(c);
        app.data.undoable.z=app.data.z(c);
        app.data.x(c)=[];
        app.data.y(c)=[];
        app.data.z(c)=[];
      end
    elseif strcmp(seltype,'extend')
      disp('Nudge up')
      app.data.z(c)=app.data.z(c)+0.01;
      app.data.selection=c;
  
      
    elseif strcmp(seltype,'alt')
      disp('Nudge down')
      app.data.z(c)=app.data.z(c)-0.01;
      app.data.selection=c;
      if 0
        % LMB, locally flatten
        % SHOULD NEVER GET HERE!  THIS CODE IS REDUNDANT BUT LEFT TO SHOW
        % HOW...
        [zfit,zfit2]=fit_plane(app.data.x(c),app.data.y(c),app.data.z(c),app.data.x,app.data.y);
        app.data.z=app.data.z-zfit2;
      end
    else
      disp('Ooh, an unusual mouse button...')
    end
    % after cropping, reset colormap
    app.view.clim=[min(app.data.z(:)) max(app.data.z(:))];

    sfReplotData
  end


%% sfReplotData
  function sfReplotData
    figure(app.view.h_fig);
    app.view.h_plot=plot3c(app.data.x,app.data.y,app.data.z,app.data.z,'.');
    xlabel('x');ylabel('y');zlabel('z');
    set(app.view.h_plot,'markersize',app.view.markersize(app.view.msize_selector));


    view(app.view.viewvector(app.view.viewaxis_selector,:));
    set(gca,'cameraupvector',app.view.cameraupvector(app.view.viewaxis_selector,:));
    set(gca,'dataaspectratio',app.view.aspectratio(app.view.ar_selector,:));
    set(gca,'clim',app.view.clim);
    set(gca,'xlim',app.view.axlimits(1:2),'ylim',app.view.axlimits(3:4));
    app.view.h_colorbar=colorbar;
    if app.view.showoutline==1
      hold on
      p=plot3(app.data.xOut,app.data.yOut,(0.1+mean(app.data.z(:)))*ones(size(app.data.xOut)),'k-')
      set(p,'linewidth',1)
      hold off
    end
  end

%% sfSetupVariables
  function sfSetupVariables
    app.view.viewaxis_selector=1;
    app.view.viewvector=[1 0 0;0 1 0;0 0 1;-0.5 -0.5 1;1 0 0;0 1 0;0 0 1;-0.5 -0.5 1];
    app.view.cameraupvector=[0 0 1;0 0 1;0 1 0;0 0 1;0 1 0;1 0 0;1 0 0;0 0 1];

    app.view.aspectratio=[...
      1e-4 1e-4 1;...
      1e-3 1e-3 1;...
      1e-2 1e-2 1;...
      1e-1 1e-1 1;...
      1e+0 1e+0 1;...
      1e+1 1e+1 1;...
      1e+2 1e+2 1;...
      1e+3 1e+3 1;...
      1e+4 1e+4 1];


    app.view.markersize=[5 15 20 25 30];
    app.view.showoutline=0;

    app.data.z=app.data.z-fit_plane(app.data.x,app.data.y,app.data.z);

    app.view.clim=[min(app.data.z(:)) max(app.data.z(:))];


    % estimate correct aspect ratio to start with
    rx=max(app.data.x)-min(app.data.x);
    rz=max(app.data.z)-min(app.data.z);
    ratio=rx./rz;
    app.view.ar_selector=find(ratio>app.view.aspectratio(:,1),1,'last');
    app.view.msize_selector=1;
    
    app.view.zoomon=0;
    app.view.axlimits=[min(app.data.x) max(app.data.x) min(app.data.y) max(app.data.y)];
    set(gca,'xlim',app.view.axlimits(1:2),'ylim',app.view.axlimits(3:4));

  end

%% sfSetupFigure
  function sfSetupFigure
    % must start with fresh figure!
    app.view.h_fig=figure(1);
    set(gcf,'name','cmm_clean');
    set(gcf,'numbertitle','off');
    set(gcf,'toolbar','none');
    set(gcf,'menubar','none');
    close(app.view.h_fig)
    app.view.h_fig=figure(1);
    % ensure that keystrokes get captured and NOT passed to command line
    set(app.view.h_fig,'KeyPressFcn','cmm_clean(''key'')');
    set(app.view.h_fig,'WindowButtonDownFcn','cmm_clean(''mouse'')');
    
    app.view.clim=[min(app.data.z(:)) max(app.data.z(:))];

    clf
    % draw data for the first time
    app.view.h_plot=plot3c(app.data.x,app.data.y,app.data.z,app.data.z,'.');
    xlabel('x');ylabel('y');zlabel('z');
    app.view.h_ax=gca;
    set(app.view.h_plot,'markersize',app.view.markersize(app.view.msize_selector));
    set(gca,'position',[0.2 0.1 0.8 0.8])

    app.view.h_colorbar=colorbar;

    set(gcf,'pointer','crosshair');

    view(app.view.viewvector(app.view.viewaxis_selector,:));
    set(gca,'cameraupvector',app.view.cameraupvector(app.view.viewaxis_selector,:));
    set(gca,'dataaspectratio',app.view.aspectratio(app.view.ar_selector,:))
    set(gca,'xlim',app.view.axlimits(1:2),'ylim',app.view.axlimits(3:4));
    app.gui.uipanel=uipanel('units','pixels','position',[7 40 67 340]);
    
    
    bsize=[55 20];
         
    app.gui.button(1)=uicontrol(gcf,'parent',app.gui.uipanel,...
      'position',[5 300 bsize],'string','YZ',...
      'tooltipstring','Shortcut: 1',...
      'callback','set(gcf,''currentcharacter'',''1'');cmm_clean(''key'')');
    app.gui.button(2)=uicontrol(gcf,'parent',app.gui.uipanel,...
      'position',[5 275 bsize],'string','XZ',...
      'tooltipstring','Shortcut: 2',...
      'callback','set(gcf,''currentcharacter'',''2'');cmm_clean(''key'')');
    app.gui.button(3)=uicontrol(gcf,'parent',app.gui.uipanel,...
      'tooltipstring','Shortcut: 3',...
      'position',[5 250 bsize],'string','XY',...
      'callback','set(gcf,''currentcharacter'',''3'');cmm_clean(''key'')');
    app.gui.button(4)=uicontrol(gcf,'parent',app.gui.uipanel,...
      'tooltipstring','Shortcut: 4',...
      'position',[5 225 bsize],'string','Iso',...
      'callback','set(gcf,''currentcharacter'',''4'');cmm_clean(''key'')');
    app.gui.button(5)=uicontrol(gcf,'parent',app.gui.uipanel,...
      'tooltipstring','Shortcut: f',...
      'position',[5 200 bsize],'string','flatten',...
      'callback','set(gcf,''currentcharacter'',''f'');cmm_clean(''key'')');
    app.gui.button(6)=uicontrol(gcf,'parent',app.gui.uipanel,...
      'position',[5 175 bsize],'string','Aspect',...
      'tooltipstring','Shortcut: a',...
      'callback','set(gcf,''currentcharacter'',''a'');cmm_clean(''key'')');
   app.gui.button(7)=uicontrol(gcf,'parent',app.gui.uipanel,...
     'position',[5 150 bsize],'string','Marker',...
     'tooltipstring','Shortcut: m',...
     'callback','set(gcf,''currentcharacter'',''m'');cmm_clean(''key'')');
   app.gui.button(8)=uicontrol(gcf,'parent',app.gui.uipanel,...
     'position',[5 125 bsize],'string','SquareD',...
     'tooltipstring','Shortcut: r',...
     'callback','set(gcf,''currentcharacter'',''r'');cmm_clean(''key'')');
   app.gui.button(9)=uicontrol(gcf,'parent',app.gui.uipanel,...
     'position',[5 100 bsize],'string','TogOutline',...
     'tooltipstring','Shortcut: o',...
     'callback','set(gcf,''currentcharacter'',''o'');cmm_clean(''key'')');
   app.gui.button(10)=uicontrol(gcf,'parent',app.gui.uipanel,...
     'position',[5 75 bsize],'string','ColLimits',...
     'tooltipstring','Shortcut: c',...
     'callback','set(gcf,''currentcharacter'',''c'');cmm_clean(''key'')');
   app.gui.button(11)=uicontrol(gcf,'parent',app.gui.uipanel,...
     'position',[5 50 bsize],'string','WriteData',...
     'tooltipstring','Shortcut: w',...
     'callback','set(gcf,''currentcharacter'',''w'');cmm_clean(''key'')');
   app.gui.button(12)=uicontrol(gcf,'parent',app.gui.uipanel,...
     'position',[5 25 bsize],'string','Quit',...
     'tooltipstring','Shortcut: q',...
     'callback','set(gcf,''currentcharacter'',''q'');cmm_clean(''key'')');
   
  end

%% sfCycleAspectRatio
  function sfCycleAspectRatio
    app.view.ar_selector=mod(app.view.ar_selector,size(app.view.aspectratio,1))+1;
    set(app.view.h_ax,'dataaspectratio',app.view.aspectratio(app.view.ar_selector,:));
  end

%% sfCycleMarkerSize
  function sfCycleMarkerSize
    app.view.msize_selector=mod(app.view.msize_selector,length(app.view.markersize))+1;
    set(app.view.h_plot,'markersize',app.view.markersize(app.view.msize_selector));
  end

%% sfFlattenData
  function sfFlattenData
    app.data.z=app.data.z-fit_plane(app.data.x,app.data.y,app.data.z);
    % after flatten, reset the color limits
    app.view.clim=[min(app.data.z) max(app.data.z)];
    sfReplotData
  end

%% sfSetColourLimits
  function sfSetColourLimits
    app.view.clim(1)=input(['Lower value for color scale [' ...
      num2str(app.view.clim(1)) ']: ']);
    app.view.clim(2)=input(['Upper value for color scale [' ...
      num2str(app.view.clim(2)) ']: ']);
    set(app.view.h_ax,'clim',app.view.clim);
    colorbar
  end


%% sfToggleZoom
  function sfToggleZoomIn
    if app.view.zoomon
      disp('ZoomIn OFF')
      app.view.zoomon=0;
       else
      disp('ZoomIn ON')
      app.view.zoomon=1;
    end
  end

%% sfZoomOut
  function sfZoomOut
    app.view.axlimits=[min(app.data.x) max(app.data.x) min(app.data.y) max(app.data.y)];
    set(gca,'xlim',app.view.axlimits(1:2),'ylim',app.view.axlimits(3:4));
  end
%% sfSquareData
  function sfSquareData
    if ~isfield(app.data,'xOut')
      disp('Reading outline file')
      sfReadOutline
    end
    outline=[app.data.xOut app.data.yOut];
    data=[app.data.x app.data.y app.data.z];
    [outline,data]=sfRotateSquare(outline,data);
    app.data.x=data(:,1);app.data.y=data(:,2);app.data.z=data(:,3);
    app.data.xOut=outline(:,1);app.data.yOut=outline(:,2);
    dlmwrite([app.data.outlinefile(1:end-4) '_square.txt'],[app.data.xOut app.data.yOut app.data.zOut],'delimiter',' ');
    disp('Written _SQUARE data')
    sfReplotData
  end

%% sfReadOutline
  function sfReadOutline
    app.data.outlinefile=uigetfile('*.*','Select outline file');
    if isequal(app.data.outlinefile,0)
      disp('Cancelled');
    else
      [app.data.xOut,app.data.yOut,app.data.zOut]=textread(app.data.outlinefile);
    end
  end

%% sfToggleOutline
  function sfToggleOutline
    if ~isfield(app.data,'xOut')
      sfReadOutline
      if ~isfield(app.data,'xOut')
        % sfReadOutline didn't manage to read the file (cancelled?)
        return
      end
    end
    if app.view.showoutline==1
      app.view.showoutline=0;
    else
      app.view.showoutline=1;
    end
    sfReplotData
  end

%% sfUndoLastCrop
  function sfUndoLastCrop
    app.data.x=[app.data.x; app.data.undoable.x];
    app.data.y=[app.data.y; app.data.undoable.y];
    app.data.z=[app.data.z; app.data.undoable.z];
    sfReplotData
  end


%% sfWriteData
  function sfWriteData
    disp('Writing cleaner FLAT file...')
    dlmwrite([app.data.fnamedata(1:end-4) '_flat.txt'],[app.data.x app.data.y app.data.z],'delimiter',' ');
  end

%% sfQuit
  function sfQuit
    disp('Quit');
    set(app.gui.button,'enable','off')
    set(app.view.h_fig,'KeyPressFcn',[]);
    set(app.view.h_fig','WindowButtonDownFcn',[])
  end

%% sfRotateSquare
function [outline,data]=sfRotateSquare(outline,data)
  %% SQUAREDATA
  % Samples often have a nominally flat edge to them, so this assumes that
  % one of those edges is in the bottom 5% of the data, and uses that edge to
  % rotate the datasets to make the bottom horizontal.  Could go horribly
  % wrong, but was developed for railway samples and works well there.  Needs
  % to be done before centering.
  routline=max(outline(:,2))-min(outline(:,2));
  fivepercentile_outline=min(outline(:,2))+0.01*routline;
  bottomsection_outline=find(outline(:,2)<fivepercentile_outline);

  fit_outline=polyfit(outline(bottomsection_outline,1),outline(bottomsection_outline,2),1);

  % small angle approximation could be used here but no benefit
  outline_theta=-tan(fit_outline(1));

  T=[cos(outline_theta) sin(outline_theta);-sin(outline_theta) cos(outline_theta)];
  % apply T to both outline and datapoints
  outline(:,[1 2])=[outline(:,1) outline(:,2)]*T;
  data(:,[1 2])=[data(:,1) data(:,2)]*T;

end
  %%%%%%%%%%%%%%%%%%%
end
