% FIT_SPLINE.M
% Greg Johnson & Matt Fox
% April 2004
% completely rewritten June 2004 by Greg - now more general
%
% REQUIRES: Spline toolbox and exportfig

% Hmm.  Maybe the splines are NOT cubic but only parabolic?  Look at
% repx,repy (and spap2).  What does this mean???

% height range - tweak manually
%r=-100:10:100;
%%
r=input('Enter range for colour bar (microns) ([min:step:max] = [-100:10:100]): ');
if isempty(r)
  return
end

%% produce a colourbar
close all
figure(1)

[x,y]=meshgrid(r,1:2);
contourf(x,y,[r;r],[r])
set(gca,'ytick',[])
set(gca,'xtick',[r])
pos=get(gca,'position');
set(gca,'position',[pos(1) pos(2) pos(3) 0.1])
try
  system('mkdir results');
catch
  disp('Maybe the "results" directory already exists...')
end
exportfig(gcf,['results/colourbar.eps'],'format','eps2','color','cmyk','preview','none')
saveas(gcf,['results/colourbar.fig']);
print('-djpeg',['results/colourbar.jpg'])

%%



if ~exist('outlinefile','var')
  outlinefile=uigetfile('*.txt','Select the outline file');
  if outlinefile==0
    disp('Cancelled')
    clear outlinefile
    return
  end
end
[xOut,yOut,zOut]=read_three_columns(outlinefile);

if ~(exist('filename1','var') & exist('filename2','var'))
  fnames=uigetfile('*.txt','Select the data files','multiselect','on');
  filename1=fnames{1}
  filename2=fnames{2};
  %end
  [xA,yA,zA]=read_three_columns(filename1);
  [xB,yB,zB]=read_three_columns(filename2);

  % estimate point density
  pdensity=max(round(abs(10*median(diff(yA))))/10,round(abs(10*median(diff(xA))))/10);;

  if pdensity==0
    pdensity=input('Point density estimated at 0 - enter correct value: ');
  else
    fprintf('Point density estimated at %2.2f: edit this file if incorrect\n',pdensity);
  end

  disp('Fitting splines to data')

  % only take points every 'decimation' mm

  decimation=pdensity;
  xpad=0.05*(max(xOut)-min(xOut));
  ypad=0.05*(max(yOut)-min(yOut));

  repx=3;
  repy=repx;

  xrange=min(xOut)-xpad:decimation:max(xOut)+xpad;
  yrange=min(yOut)-ypad:decimation:max(yOut)+ypad;
  [xi,yi,zAi]=griddata(xA,yA,zA,xrange,yrange','nearest');
  %[zBi]=griddata(xB,yB,zB,xrange,yrange','nearest');
  zCi=(zAi+zAi)/2;

  disp('Calculating points inside boundary.  Please hold...')
  in=inpolygon(xi(:),yi(:),xOut,yOut);
  disp('Writing averaged data to ''averaged.mat''')
  av.in=in;
  av.zCi=zCi;
  av.z=av.zCi;av.z(~av.in)=NaN;
  av.xi=xi;
  av.yi=yi;
  save('averaged','av')

end


% open log of fit stats
fid=fopen('results/spline_gof.txt','wt');
fprintf(fid,'KnotSpacing(mm)\tOneSigma(um)\tRange(um)\tRMS(um)\n');
close all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% KNOT SPACING SET HERE 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

spacingrange=input('Specify minimum and maximum point spacing (mm) [x y]: ');
if 0
  spacingset=[spacingrange(2)];
  minspacing=max(pdensity*2,spacingrange(1));  % can't fit splines with density less than 2x data density
  while spacingset(end)>minspacing
    spacingset(end+1)=spacingset(end)/2;
  end
  %spacingset=spacingset(end:-1:1);
else
  % new attempt here ** greg, april 2005)
  disp('New range done here...')
  spacingset=[spacingrange(1):spacingrange(2)];
end
midpointx=mean([min(xOut) max(xOut)]);
midpointy=mean([min(yOut) max(yOut)]);

%%%%
if 0
  spacingset=[];
  n=2;
  tmpxrange=max(xrange)-min(xrange);
  tmpyrange=max(yrange)-min(yrange);
  minrange=min(tmpxrange,tmpyrange);
  while minrange/n>2*pdensity
    spacingset(end+1)=minrange/n;
    n=n+1;
  end
  spacingset=spacingset(:);
end
%%%%%%%%%%%%%%%%%%%%%
for n=1:length(spacingset)
  
  knotspacex=spacingset(n);
  %knotspacex=pdensity*2;
  knotspacey=knotspacex;
  %knotspacey=pdensity*4;

  fprintf('Spline knot spacing: X=%2.2f mm  Y=%2.2f mm\n',knotspacex,knotspacey);
  
  tmp=midpointx;
  while tmp(end)<max(xOut)
    tmp(end+1)=tmp(end)+knotspacex;
  end
  rhrangex=tmp;

  tmp=midpointx;
  while tmp(end)>min(xOut)
    tmp(end+1)=tmp(end)-knotspacex;
  end
  lhrangex=tmp(end:-1:2);

  tmpx2=[lhrangex rhrangex];
  %%%%%%
  tmp=midpointy;
  while tmp(end)<max(yOut)
    tmp(end+1)=tmp(end)+knotspacey;
  end
  rhrangey=tmp;

  tmp=midpointy;
  while tmp(end)>min(yOut)
    tmp(end+1)=tmp(end)-knotspacey;
  end
  lhrangey=tmp(end:-1:2);

  tmpy2=[lhrangey rhrangey];

  spline=spap2({augknt(tmpx2,repx),
    augknt(tmpy2,repy)},...
    [repx repy],{xrange,yrange},zCi');

  zerror=fnval(spline,[xi(:) yi(:)]');
  zerror=zCi-reshape(zerror,size(xi));

  zOutC=fnval(spline,[xOut yOut]');

  %%%%%%%%%%%%%%%%%%
  if 0
    figure(1)
    clf
    surf(xi,yi,zCi,'facecolor','none','edgecolor',[.8 .8 .8],'edgealpha',0.7)
    hold on
    fnplt(spline)
    plot3(spline.knots{1},zeros(size(spline.knots{1})),ones(size(spline.knots{1}))*max(zB(:)),'ro')
    plot3(zeros(size(spline.knots{2})),spline.knots{2},ones(size(spline.knots{2}))*max(zB(:)),'ro')
    view([1 0 0])
    set(gca,'dataaspectratio',[500 500 1])

    title('Mesh of averaged point clouds (gray) with spline fit');
  end
  %%
  %%%%%%%%%%%%%%%%%%%%%
  if 0
    figure(2)
    clf

    fnplt(spline)
    hold on
    plot3(xOut,yOut,zOutC,'.')
    plot3(spline.knots{1},zeros(size(spline.knots{1})),ones(size(spline.knots{1}))*max(zB(:)),'ko')
    plot3(zeros(size(spline.knots{2})),spline.knots{2},ones(size(spline.knots{2}))*max(zB(:)),'ko')

    title('Fitted spline, outline of sample')
    set(gca,'clim',[-0.04 0.04])
    view([0 0 1])
  end
  %%

  zerr2=zerror;
  zerr2(~in)=NaN;

  if 0
    figure(3)
    clf
    mesh(xi,yi,zerr2);view([0 0 1]),axis image
    %hold on
    %plot3(xOut,yOut,zOut,'.')
    title('Error between fitted spline and averaged surfaces')
    xlabel('mm');
    ylabel('mm');
    zlabel('mm');
  end
  e=zerror(in)*1000;
  sperror(n,:)=e;
  onesigma=std(e);
  range1=max(e)-min(e);
  rms=sqrt(mean(e.^2));
  fprintf('STD error: %2.2fum range: %2.2fum RMS: %2.2fum\n\n',onesigma,range1,rms);
  
  fprintf(fid,'%02d\t\t%2.2f\t\t%2.2f\t\t%2.2f\n',spacingset(n),onesigma,range1,rms);
  
  %%
  figure(5)
  % contour plot of spline'd surface
  a=yi;
  b=xi;
  c=reshape(fnval(spline,[xi(:) yi(:)]'),size(xi));
  c(~in)=NaN;
  contourf(a,b,c*1000,r)
  set(gca,'clim',[min(r) max(r)])
  set(gca,'dataaspectratio',[1000 1000 1]);
  axis tight
  xlabel('mm'),ylabel('mm')
  disp('MANUAL COLOUR LIMITS - CHANGE THIS IN FIT_SPLINE.M')
  
 % colorbar('horizontal','gridlinestyle','-','xgrid','on')
  
  if floor(spacingset(n))==0
    fname=sprintf('spline00_%03.0f',1000*spacingset(n));
  else
    fname=sprintf('spline%02d_%03.0f',[floor(spacingset(n)) 1000*rem(spacingset(n),floor(spacingset(n)))]);
  end
  titlestr=sprintf('Surface height of averaged data points (um) - %s',fname);
  title(titlestr,'interpreter','none','fontsize',15)


  disp(['Filename: ',fname]);
  exportfig(gcf,['results/' fname '.eps'],'format','eps2','color','cmyk','preview','none')
  saveas(gcf,['results/' fname '.fig']);
  print('-djpeg',['results/' fname '.jpg'])
  %%
  save([fname '.mat'],'spline')
end
  
fclose(fid);


%%
figure
%%
clf
numplots=size(sperror,1);
clear ax hp
for n=numplots:-1:1
  subplot(numplots/2,2,numplots+1-n)
  hp(n)=plot(sperror(n,:),'.');
  axis([-inf inf -5 5])
  ax(n)=gca;
  title([num2str(spacingset(n)) 'mm'])
end

hlink=linkprop(ax,{'xlim','ylim'});
%set(ax,'fontsize',14)
set(hp,'markersize',1)
exportfig(gcf,'results/residuals_all.eps','format','eps2','color','cmyk','preview','none')
print('-djpeg',['results/residuals_all.jpg']);

%axis([5200 5500 -5 5])
%set(hp,'markersize',6)
%exportfig(gcf,'results/residuals_zoom.eps','format','eps2','color','cmyk','preview','none')

