function gen_cmmscan(outlinefile,ztop,partprogram,measurementfilename,pitch,type)
% generates a grid of points within part boundary
% Usage:
% gen_cmmscan(outlinefile,ztop,partprogram,measurementfilename,pitch)

if nargin~=6
  help(mfilename)
  return
end
tic
close all
pitchx=pitch;
pitchy=pitch;

[x,y,z]=textread(outlinefile);
plot3(x,y,z,'.-','markersize',1);
axis equal

bbox=[min(x) min(y) max(x) max(y)];  % left front back right
hold on
meanz=mean(z);

startx=bbox(1);
starty=bbox(2);
endx=bbox(3);
endy=bbox(4);
zclearance=ztop+100
zmeasurement=ztop-2;
zstandoff=ztop+2;


fid=fopen([partprogram '.agw'],'wt');
fprintf(fid,'$$ Matlab generated scan lines\n');
fprintf(fid,'\n');
fprintf(fid,['FILNAM/ "' partprogram '"\n']);
fprintf(fid,'\n');
fprintf(fid,'SNSLCT/1\n');
fprintf(fid,'DATSET/MCS\n');
%fprintf(fid,'RECALL/DA,27\n');
fprintf(fid,'CNCON / MESVEL = DEFALT, POSVEL = HIGH, APPRCH = 0.5\n');
fprintf(fid,'\n');

%first half of sample
%xrange=startx-2:pitchx:endx+2;

%reverse for 2nd half of sample
xrange=endx+2:-pitchx:startx-2;
yrange=starty-2:pitchy:endy+2;


xpos=zeros(1,length(xrange)*length(yrange));
ypos=zeros(1,length(xrange)*length(yrange));

n=1;
for a=xrange
  for b=yrange
    xpos(n)=a;
    ypos(n)=b;
    n=n+1;
  end
end
disp('Processing boundary');

if strcmp(upper(type),'CONTINUOUS')
  % attempt scanning measurements
  % calculate start and end points for each contour
  disp('Continuous scanning')

  
  convex_ndx=convhull(x,y);
  inside=inpolygon(xpos,ypos,x(convex_ndx),y(convex_ndx));
  disp('Generating code...');
  
  edges=find(diff(inside)~=0);
  %plot3(xpos(edges),ypos(edges),repmat(meanz,size(edges)),'go')
  xtmp=xpos(edges);ytmp=ypos(edges);
   measspeed=6;
  meassafe=1;
  contourndx=1;
  for n=1:2:length(edges) %edges should always be an even number
    plot3(xtmp(n:n+1),ytmp(n:n+1),[meanz meanz],'r-o')
    hold on
    fprintf(fid,'DRIVE/ZAXIS, %f\n',meanz+6);  
    fprintf(fid,'GOTO/CART,%f,%f,%f\n',xtmp(n),starty-4,meanz+6);    %constant start y  
    fprintf(fid,'CONTOUR/MEAS, "Scan", NUMBER=1\n');
    fprintf(fid,'AUTOM/CONTOUR,$\n');
    fprintf(fid,'YZPLAN,$\n');
    fprintf(fid,'CART,$\n');
    fprintf(fid,'%3.3f,%3.3f,%3.3f,$\n',xtmp(n),starty-4,meanz); %constant start y
    fprintf(fid,'%3.3f,%3.3f,%3.3f,$\n',xtmp(n+1),starty-3,meanz); %constant start y
    fprintf(fid,'VECCOMP,   0.0000,   1.0000,  0.0000,$\n');
    fprintf(fid,'PITCH =%3.3f,$\n',pitch);
    fprintf(fid,'SCNVEL=%3.3f,$\n',measspeed);
    fprintf(fid,'SAFETY=%3.3f,$\n',meassafe);
    fprintf(fid,'LEFT,$\n');
    fprintf(fid,'IGNORE_1ST,$\n');
    %fprintf(fid,'NOCOMP,$\n'); % Comment this line for COMPON
    fprintf(fid,'DISPLACEMENT=   0.2500\n');
    % fprintf(fid,'PATCH = 56\n');
    fprintf(fid,'ENDMES\n');
    if contourndx==1  % overwrite any exisiting file
      fprintf(fid,['CONTOUR/EXPORT,"Scan",NUMBER=1,"' measurementfilename '"\n']);
      contourndx=0;
    else  % append to existing file
      fprintf(fid,['CONTOUR/EXPORT,"Scan",NUMBER=1,"' measurementfilename '",APPEND\n']);
    end
    
  end

  
elseif strcmp(upper(type),'DISCRETE')
  inside=inpolygon(xpos,ypos,x,y);

  
  fprintf(fid,'CONTOUR/MEAS, "Scan", NUMBER=1\n');
  fprintf(fid,'\n');
  fprintf(fid,'ASSIGN/''ZMEASUREMENT''="%f",FIXP=4\n',zmeasurement);
  fprintf(fid,'ASSIGN/''ZCLEAR''="%f",FIXP=4\n',zclearance);
  fprintf(fid,'ASSIGN/''ZSTANDOFF''="%f",FIXP=4\n',zstandoff);
  fprintf(fid,'DRIVE/ZAXIS,''ZCLEAR''\n');

  fprintf(fid,'\n');
  points=1;
  contourndx=1;
  totalnumlines=0;

  disp('Generating code...');
  for point=find(inside==1)
    plot3(xpos(point),ypos(point),meanz,'r.','markersize',1);hold on

    fprintf(fid,'GOTO/CART, %3.3f, %3.3f, ''ZSTANDOFF''\n',...
      xpos(point),ypos(point));
    fprintf(fid,'PTMEAS/CART,%3.3f,%3.3f,''ZMEASUREMENT'',VECCOMP,0,0,-1\n',...
      xpos(point),ypos(point));
    totalnumlines=totalnumlines+2;
    if points>100
      %disp(['Contour ' num2str(contourndx)]);
      drawnow
      fprintf(fid,'ENDMES\n');
      if contourndx==1  % overwrite any exisiting file
        fprintf(fid,['CONTOUR/EXPORT,"Scan",NUMBER=1,"' measurementfilename '"\n']);
      else  % append to existing file
        fprintf(fid,['CONTOUR/EXPORT,"Scan",NUMBER=1,"' measurementfilename '",APPEND\n']);
      end
      fprintf(fid,'CONTOUR/MEAS, "Scan", NUMBER=1\n');
      points=1;
      contourndx=contourndx+1;
      totalnumlines=totalnumlines+2;
    else
      points=points+1;
    end

  end

  if points~=1
    disp(['Contour ' num2str(contourndx)]);
    fprintf(fid,'ENDMES\n');
    if contourndx==1  % overwrite any exisiting file
      fprintf(fid,['CONTOUR/EXPORT,"Scan",NUMBER=1,"' measurementfilename '"\n']);
    else  % append to existing file
      fprintf(fid,['CONTOUR/EXPORT,"Scan",NUMBER=1,"' measurementfilename '",APPEND\n']);
    end
  end
  fprintf('Number of lines in program ~= %d\n',totalnumlines);

else
  disp('Scan type can be ''discrete'' or ''continuous''');
  return
end

axis equal
fprintf(fid,'ENDFIL\n');
fclose(fid);


toc

